@@ -2616,6 +2616,15 @@ bus_driver_handle_message (DBusConnection *connection,
       BusContext *context;
       DBusConnection *systemd;
 
+      /* This is a directed signal, not a method call, so the log message
+       * is a little weird (it talks about "calling" ActivationFailure),
+       * but it's close enough */
+      if (!bus_driver_check_caller_is_privileged (connection,
+                                                  transaction,
+                                                  message,
+                                                  error))
+        return FALSE;
+
       context = bus_connection_get_context (connection);
       systemd = bus_driver_get_owner_of_name (connection,
           "org.freedesktop.systemd1");
@@ -2634,6 +2643,14 @@ bus_driver_handle_message (DBusConnection *connection,
           return TRUE;
         }
 
+      if (!bus_context_get_systemd_activation (context))
+        {
+          bus_context_log (context, DBUS_SYSTEM_LOG_WARNING,
+                           "Ignoring unexpected ActivationFailure message "
+                           "while not using systemd activation");
+          return FALSE;
+        }
+
       return dbus_activation_systemd_failure(bus_context_get_activation(context), message);
     }
 
