@@ -76,6 +76,7 @@ bus_registry_new (BusContext *context)
 {
   BusRegistry *registry;
 
+  _dbus_assert (context);
   registry = dbus_new0 (BusRegistry, 1);
   if (registry == NULL)
     return NULL;
@@ -461,8 +462,7 @@ bus_registry_acquire_service (BusRegistry      *registry,
     }
 
   if (!bus_apparmor_allows_acquire_service (connection,
-                                            (registry->context ?
-                                             bus_context_get_type (registry->context) : NULL),
+                                            bus_context_get_type (registry->context),
                                             _dbus_string_get_const_data (service_name), error))
     goto out;
   
