@@ -77,8 +77,8 @@ int _dbus_connect_exec (const char     *path,
                         char *const    argv[],
                         DBusError      *error);
 
-int _dbus_listen_systemd_sockets (int       **fd,
-                                 DBusError *error);
+int _dbus_listen_systemd_sockets (DBusSocket  **fd,
+                                  DBusError    *error);
 
 dbus_bool_t _dbus_read_credentials (int               client_fd,
                                     DBusCredentials  *credentials,
@@ -138,7 +138,6 @@ dbus_bool_t _dbus_group_info_fill_gid (DBusGroupInfo    *info,
                                        DBusError        *error);
 void        _dbus_group_info_free     (DBusGroupInfo    *info);
 
-dbus_uid_t    _dbus_getuid (void);
 DBUS_PRIVATE_EXPORT
 dbus_uid_t    _dbus_geteuid (void);
 
@@ -148,13 +147,24 @@ dbus_bool_t _dbus_parse_uid (const DBusString  *uid_str,
 DBUS_PRIVATE_EXPORT
 void _dbus_close_all (void);
 
-dbus_bool_t _dbus_append_address_from_socket (int         fd,
+dbus_bool_t _dbus_append_address_from_socket (DBusSocket  fd,
                                               DBusString *address,
                                               DBusError  *error);
 
 DBUS_PRIVATE_EXPORT
 void _dbus_fd_set_close_on_exec (int fd);
 
+typedef enum
+{
+  DBUS_FORCE_STDIN_NULL = (1 << 0),
+  DBUS_FORCE_STDOUT_NULL = (1 << 1),
+  DBUS_FORCE_STDERR_NULL = (1 << 2)
+} DBusEnsureStandardFdsFlags;
+
+DBUS_PRIVATE_EXPORT
+dbus_bool_t _dbus_ensure_standard_fds (DBusEnsureStandardFdsFlags   flags,
+                                       const char                 **error_str_p);
+
 /** @} */
 
 DBUS_END_DECLS
