@@ -2,6 +2,7 @@
  *
  * Author: Simon McVittie <simon.mcvittie@collabora.co.uk>
  * Copyright © 2010-2011 Nokia Corporation
+ * Copyright © 2015 Collabora Ltd.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation files
@@ -26,8 +27,18 @@
 
 #include <config.h>
 
+#include <errno.h>
+#include <string.h>
+
+#include <dbus/dbus.h>
+
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include "test-utils-glib.h"
 
+#include <string.h>
+
 #ifdef DBUS_UNIX
 # include <unistd.h>
 # include <sys/types.h>
@@ -73,6 +84,9 @@ typedef struct {
     DBusConnection *right_conn;
     gboolean right_conn_echo;
     gboolean wait_forever_called;
+
+    gchar *tmp_runtime_dir;
+    gchar *saved_runtime_dir;
 } Fixture;
 
 static DBusHandlerResult
@@ -110,6 +124,7 @@ typedef struct {
     const char *bug_ref;
     guint min_messages;
     const char *config_file;
+    enum { SPECIFY_ADDRESS = 0, RELY_ON_DEFAULT } connect_mode;
 } Config;
 
 static void
@@ -123,6 +138,17 @@ setup (Fixture *f,
   f->ge = NULL;
   dbus_error_init (&f->e);
 
+  if (config != NULL && config->connect_mode == RELY_ON_DEFAULT)
+    {
+      /* this is chosen to be something needing escaping */
+      f->tmp_runtime_dir = g_dir_make_tmp ("dbus=daemon=test.XXXXXX", &f->ge);
+      g_assert_no_error (f->ge);
+
+      /* we're relying on being single-threaded for this to be safe */
+      f->saved_runtime_dir = g_strdup (g_getenv ("XDG_RUNTIME_DIR"));
+      g_setenv ("XDG_RUNTIME_DIR", f->tmp_runtime_dir, TRUE);
+    }
+
   address = test_get_dbus_daemon (config ? config->config_file : NULL,
                                   TEST_USER_ME,
                                   &f->daemon_pid);
@@ -134,7 +160,22 @@ setup (Fixture *f,
     }
 
   f->left_conn = test_connect_to_bus (f->ctx, address);
-  f->right_conn = test_connect_to_bus (f->ctx, address);
+
+  if (config != NULL && config->connect_mode == RELY_ON_DEFAULT)
+    {
+      /* use the default bus for the echo service ("right"), to check that
+       * it ends up on the same bus as the client ("left") */
+      f->right_conn = dbus_bus_get_private (DBUS_BUS_SESSION, &f->e);
+      test_assert_no_error (&f->e);
+
+      if (!test_connection_setup (f->ctx, f->right_conn))
+        g_error ("OOM");
+    }
+  else
+    {
+      f->right_conn = test_connect_to_bus (f->ctx, address);
+    }
+
   g_free (address);
 }
 
@@ -305,7 +346,8 @@ test_creds (Fixture *f,
   enum {
       SEEN_UNIX_USER = 1,
       SEEN_PID = 2,
-      SEEN_WINDOWS_SID = 4
+      SEEN_WINDOWS_SID = 4,
+      SEEN_LINUX_SECURITY_LABEL = 8
   } seen = 0;
 
   if (m == NULL)
@@ -364,11 +406,32 @@ test_creds (Fixture *f,
           g_assert_cmpuint (dbus_message_iter_get_arg_type (&var_iter), ==,
               DBUS_TYPE_UINT32);
           dbus_message_iter_get_basic (&var_iter, &u32);
-          g_message ("%s of this process is %u", name, u32);
+          g_test_message ("%s of this process is %u", name, u32);
           g_assert_cmpuint (u32, ==, geteuid ());
           seen |= SEEN_UNIX_USER;
 #else
           g_assert_not_reached ();
+#endif
+        }
+      else if (g_strcmp0 (name, "WindowsSID") == 0)
+        {
+#ifdef G_OS_WIN32
+          gchar *sid;
+          char *self_sid;
+
+          g_assert (!(seen & SEEN_WINDOWS_SID));
+          g_assert_cmpuint (dbus_message_iter_get_arg_type (&var_iter), ==,
+              DBUS_TYPE_STRING);
+          dbus_message_iter_get_basic (&var_iter, &sid);
+          g_test_message ("%s of this process is %s", name, sid);
+          if (_dbus_getsid (&self_sid, 0))
+            {
+              g_assert_cmpstr (self_sid, ==, sid);
+              LocalFree(self_sid);
+            }
+          seen |= SEEN_WINDOWS_SID;
+#else
+          g_assert_not_reached ();
 #endif
         }
       else if (g_strcmp0 (name, "ProcessID") == 0)
@@ -379,7 +442,7 @@ test_creds (Fixture *f,
           g_assert_cmpuint (dbus_message_iter_get_arg_type (&var_iter), ==,
               DBUS_TYPE_UINT32);
           dbus_message_iter_get_basic (&var_iter, &u32);
-          g_message ("%s of this process is %u", name, u32);
+          g_test_message ("%s of this process is %u", name, u32);
 #ifdef G_OS_UNIX
           g_assert_cmpuint (u32, ==, getpid ());
 #elif defined(G_OS_WIN32)
@@ -389,6 +452,27 @@ test_creds (Fixture *f,
 #endif
           seen |= SEEN_PID;
         }
+      else if (g_strcmp0 (name, "LinuxSecurityLabel") == 0)
+        {
+#ifdef __linux__
+          gchar *label;
+          int len;
+          DBusMessageIter ay_iter;
+
+          g_assert (!(seen & SEEN_LINUX_SECURITY_LABEL));
+          g_assert_cmpuint (dbus_message_iter_get_arg_type (&var_iter), ==,
+              DBUS_TYPE_ARRAY);
+          dbus_message_iter_recurse (&var_iter, &ay_iter);
+          g_assert_cmpuint (dbus_message_iter_get_arg_type (&ay_iter), ==,
+              DBUS_TYPE_BYTE);
+          dbus_message_iter_get_fixed_array (&ay_iter, &label, &len);
+          g_test_message ("%s of this process is %s", name, label);
+          g_assert_cmpuint (strlen (label) + 1, ==, len);
+          seen |= SEEN_LINUX_SECURITY_LABEL;
+#else
+          g_assert_not_reached ();
+#endif
+        }
 
       dbus_message_iter_next (&arr_iter);
     }
@@ -402,9 +486,7 @@ test_creds (Fixture *f,
 #endif
 
 #ifdef G_OS_WIN32
-  /* FIXME: when implemented:
   g_assert (seen & SEEN_WINDOWS_SID);
-   */
 #endif
 }
 
@@ -444,14 +526,25 @@ test_processid (Fixture *f,
   while (m == NULL)
     test_main_context_iterate (f->ctx, TRUE);
 
-  if (dbus_message_get_args (m, &error,
+  if (dbus_set_error_from_message (&error, m))
+    {
+      g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNIX_PROCESS_ID_UNKNOWN);
+
+#ifdef PID_SHOULD_WORK
+      g_error ("Expected pid to be passed, but got %s: %s",
+          error.name, error.message);
+#endif
+
+      dbus_error_free (&error);
+    }
+  else if (dbus_message_get_args (m, &error,
         DBUS_TYPE_UINT32, &pid,
         DBUS_TYPE_INVALID))
     {
       g_assert_cmpstr (dbus_message_get_signature (m), ==, "u");
       test_assert_no_error (&error);
 
-      g_message ("GetConnectionUnixProcessID returned %u", pid);
+      g_test_message ("GetConnectionUnixProcessID returned %u", pid);
 
 #ifdef G_OS_UNIX
       g_assert_cmpuint (pid, ==, getpid ());
@@ -463,14 +556,7 @@ test_processid (Fixture *f,
     }
   else
     {
-      g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNIX_PROCESS_ID_UNKNOWN);
-
-#ifdef PID_SHOULD_WORK
-      g_error ("Expected pid to be passed, but got %s: %s",
-          error.name, error.message);
-#endif
-
-      dbus_error_free (&error);
+      g_error ("Unexpected error: %s: %s", error.name, error.message);
     }
 }
 
@@ -593,17 +679,46 @@ teardown (Fixture *f,
       f->daemon_pid = 0;
     }
 
+  if (f->tmp_runtime_dir != NULL)
+    {
+      gchar *path;
+
+      /* the socket may exist */
+      path = g_strdup_printf ("%s/bus", f->tmp_runtime_dir);
+      g_assert (g_remove (path) == 0 || errno == ENOENT);
+      g_free (path);
+      /* there shouldn't be anything else in there */
+      g_assert_cmpint (g_rmdir (f->tmp_runtime_dir), ==, 0);
+
+      /* we're relying on being single-threaded for this to be safe */
+      if (f->saved_runtime_dir != NULL)
+        g_setenv ("XDG_RUNTIME_DIR", f->saved_runtime_dir, TRUE);
+      else
+        g_unsetenv ("XDG_RUNTIME_DIR");
+      g_free (f->saved_runtime_dir);
+      g_free (f->tmp_runtime_dir);
+    }
+
   test_main_context_unref (f->ctx);
 }
 
 static Config limited_config = {
-    "34393", 10000, "valid-config-files/incoming-limit.conf"
+    "34393", 10000, "valid-config-files/incoming-limit.conf",
+    SPECIFY_ADDRESS
 };
 
 static Config finite_timeout_config = {
-    NULL, 1, "valid-config-files/finite-timeout.conf"
+    NULL, 1, "valid-config-files/finite-timeout.conf",
+    SPECIFY_ADDRESS
 };
 
+#ifdef DBUS_UNIX
+static Config listen_unix_runtime_config = {
+    "61303", 1, "valid-config-files/listen-unix-runtime.conf",
+    RELY_ON_DEFAULT
+};
+#endif
+
 int
 main (int argc,
     char **argv)
@@ -621,6 +736,13 @@ main (int argc,
   g_test_add ("/processid", Fixture, NULL, setup, test_processid, teardown);
   g_test_add ("/canonical-path/uae", Fixture, NULL,
       setup, test_canonical_path_uae, teardown);
+#ifdef DBUS_UNIX
+  /* We can't test this in loopback.c with the rest of unix:runtime=yes,
+   * because dbus_bus_get[_private] is the only way to use the default,
+   * and that blocks on a round-trip to the dbus-daemon */
+  g_test_add ("/unix-runtime-is-default", Fixture, &listen_unix_runtime_config,
+      setup, test_echo, teardown);
+#endif
 
   return g_test_run ();
 }
